<?php

/**
 * Implements hook_drush_command().
 */
function drushify_drush_command() {
  $items = array();

  $items['drushify'] = array(
    'description' => "Generate a Drush commandfile for the module named on the command line.",
    'arguments' => array(
      'module' => 'Name of module that needs a Drush command.  Required.',
      'command' => 'Name of the Drush command to create.  Optional; default is the same as the module name.'
    ),
    'options' => array(
      'option-name' => array(
        'description' => 'Description of option.',
        'example-value' => 'Example values for option, if required.',
      ),
    ),
    'examples' => array(
      'drush @site drushify module command' => 'Create a Drush commandfile with a single command in the context of the specified module.',
      'drush @none drushify drushextension command' => 'Create a Drush extension that is not part of any module.',
    ),
    'aliases' => array('ify'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH,
  );

  // Commandfiles may also add topics.  These will appear in
  // the list of topics when `drush topic` is executed.
  $items['docs-drushify'] = array(
    'description' => 'Description of command topic.',
    'hidden' => TRUE,
    'topic' => TRUE,
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH,
    'callback' => 'drush_print_file',
    'callback arguments' => array(dirname(__FILE__) . '/README.txt'),
  );

  return $items;
}

/**
 * Implements hook_drush_help().
 *
 * @param
 *   A string with the help section (prepend with 'drush:')
 *
 * @return
 *   A string with the help text for your command.
 */
function drushify_drush_help($section) {
  switch ($section) {
    case 'drush:drushify':
      return dt("The drushify command creates a Drush command file for the module named on the command line.");
    // The 'title' meta item is used to name a group of
    // commands in `drush help`.  If a title is not defined,
    // the default is "All commands in ___", with the
    // specific name of the commandfile (e.g. drushify).
    // Command files with less than four commands will
    // be placed in the "Other commands" section, _unless_
    // they define a title.  It is therefore preferable
    // to not define a title unless the file defines a lot
    // of commands.
    case 'meta:drushify:title':
      return dt("drushify commands");
    // The 'summary' meta item is displayed in `drush help --filter`,
    // and is used to give a general idea what the commands in this
    // command file do, and what they have in common.
    case 'meta:drushify:summary':
      return dt("Summary of all commands in this command group.");
  }
}

/**
 * Implementation of drush_hook_COMMAND().
 */
function drush_drushify($module_name, $command_name = NULL) {
  // If we have no site, then we will presume that the
  // 'module name' is a standalone Drush extension. Otherwise,
  // we will insist that it must be a module name.
  if (drush_get_context('DRUSH_SELECTED_DRUPAL_ROOT') == NULL) {
    $drush_extension_path = '.drush/';
    $commandfile = $module_name;
    $commandfile_path = $drush_extension_path . '/' . $module_name . '/' . $commandfile . '.drush.inc';
    $home = drush_server_home();
    $filepath = $home . '/' . $commandfile_path;
  }
  else {
    drush_bootstrap_max();
    $extensions = drush_get_extensions();

    if (!array_key_exists($module_name, $extensions)) {
      return drush_set_error('DRUSHIFY_MODULE_NOT_FOUND', dt("No module named !module could be found.", array('!module' => $module_name)));
    }
    $info = $extensions[$module_name];
    $module_path = $info->filename;
    $path_parts = pathinfo($module_path);
    $commandfile = $path_parts['filename']; // 'basename' without extension
    $commandfile_path = $path_parts['dirname'] . '/' . $commandfile . '.drush.inc';
    $filepath = DRUPAL_ROOT . '/' . $commandfile_path;
  }

  // Overwriting not allowed.
  if (file_exists($filepath)) {
    return drush_set_error('DRUSH_DRUSHIFY_COMMANDFILE_EXISTS', dt("Drush command already exists at !path.", array('!path' => $filepath)));
  }

  // If the user did not specify a command name, then assume that the
  // command name should be the same as the module / extension name.
  if (!isset($command_name)) {
    $command_name = strtr($module_name, '_', '-');
  }

  // The "command function" is the command name with dashes
  // converted into underscores.
  $command_function = strtr($command_name, '-', '_');
  // The shortened command name pattern is 'foo' if the
  // commandfile is 'foo' and the command name is also 'foo',
  // and it is 'foo_bar' if the commandfile is 'foo' and the
  // command name is 'bar'.  This accounts for the Drush
  // convention of renaming the command hook 'drush_foo_foo()'
  // to simply 'drush_foo()'.
  if ($commandfile == $command_function) {
    $shortened_command_name_pattern = $commandfile;
  }
  else {
    $shortened_command_name_pattern = $commandfile . '_' . $command_function;
  }

  // Set up the replacements that we'll be doing.
  $replacements = array(
    '[MODULE]' => $module_name,
    '[COMMANDFILE]' => $commandfile,
    '[COMMAND_NAME]' => $command_name,
    '[COMMAND_FUNCTION]' => $command_function,
    '[SHORTENED_COMMAND_NAME_PATTERN]' => $shortened_command_name_pattern,
    '[LIBRARY]' => 'LIBRARYNAME',
    '[SHORTCUT]' => 'shortcut',
    );

  // Load the base template
  $template = dirname(__FILE__) . '/templates/commandfile-' . DRUSH_MAJOR_VERSION . '.tmpl';
  if (!file_exists($template)) {
    $template = dirname(__FILE__) . '/templates/commandfile.tmpl';
  }
  $drush_command = file_get_contents($template);

  // Create the Drush command by applying the replacements
  // to the template.
  $drush_command = str_replace(array_keys($replacements), array_values($replacements), $drush_command);

  // Write the Drush command to disk.
  drush_mkdir(dirname($filepath));
  file_put_contents($filepath, $drush_command);

  // Clear the Drush commandfile cache; if we do not, then Drush
  // will not see the new commandfile we just created.
  drush_invoke_process('@none', 'cache-clear', array('drush'));

  // Open the file we created in the editor.
  $exec = drush_get_editor();
  return drush_shell_exec_interactive($exec, $filepath, $filepath);
}
